﻿Class WavePlayer

    ' =====================================================================================
    '  PRIVATE VARS AND FUNCTIONS
    ' =====================================================================================
    Private ao As AudioOut = New AudioOut
    Private m_Frequency As Single = 0
    Private m_OutLevel As Double = -120
    Private m_SamplesPerSec As Int32
    Private m_kdb As Double = 8.685889638
    Private m_oldfreq As Single
    Private m_phase As Int32
    Private m_WaveTable() As Int16
    Private m_WaveType As WaveTypes
    Private m_randomInt As Int32
    Private m_randomFloat As Single
    Private m_Vout As Double

    Private Sub InitAll()
        If m_WaveTable Is Nothing OrElse m_SamplesPerSec <> m_WaveTable.Length Then
            ReDim m_WaveTable(m_SamplesPerSec - 1)
            m_oldfreq = m_Frequency
            m_phase = 0
        End If
        m_Vout = Math.Exp(m_OutLevel / m_kdb)
        Dim k32767 As Int32 = CInt(m_Vout * 32767)
        Select Case m_WaveType
            Case WaveTypes.Sine
                Dim k_pi As Double = (Math.PI * 2) / m_WaveTable.Length
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CShort(Math.Sin(i * k_pi) * k32767)
                Next
            Case WaveTypes.Triangle
                For i As Int32 = 0 To m_WaveTable.Length \ 4 - 1
                    m_WaveTable(i) = CShort(k32767 * 4 * i / m_WaveTable.Length)
                Next
                For i As Int32 = m_WaveTable.Length \ 4 To m_WaveTable.Length * 3 \ 4 - 1
                    m_WaveTable(i) = CShort(k32767 * (2 - (4 * i / m_WaveTable.Length)))
                Next
                For i As Int32 = m_WaveTable.Length * 3 \ 4 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CShort(k32767 * (4 * i / m_WaveTable.Length - 4))
                Next
            Case WaveTypes.RampPos
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CShort(k32767 * (2 * (1 - i / m_WaveTable.Length) - 1))
                Next
            Case WaveTypes.RampNeg
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CShort(k32767 * (2 * i / m_WaveTable.Length - 1))
                Next
            Case WaveTypes.Square
                For i As Int32 = 0 To m_WaveTable.Length \ 2 - 1
                    m_WaveTable(i) = CShort(k32767)
                Next
                For i As Int32 = m_WaveTable.Length \ 2 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CShort(-k32767)
                Next
        End Select
    End Sub

    Private Sub FillBuffer(ByRef data() As Int16)
        Select Case m_WaveType
            Case WaveTypes.Noise
                If m_randomInt = 0 Then m_randomInt = 1
                For i As Int32 = 0 To data.Length - 1
                    ' ---------------------------------------------------- DAA random method
                    If (m_randomInt And &H800000) <> 0 Then
                        m_randomInt = (m_randomInt << 1) Xor &H1D872B41
                    Else
                        m_randomInt = m_randomInt << 1
                    End If
                    ' ---------------------------------------------------- limit randomFloat value
                    m_randomFloat += m_randomInt * 0.0002F
                    If Math.Abs(m_randomFloat) > 1000000000.0 Then m_randomFloat = 0
                    m_randomFloat = CInt(m_randomFloat) And 32767
                    ' ---------------------------------------------------- set the value
                    data(i) = CShort(m_Vout * m_randomFloat)
                Next
            Case Else
                If m_Frequency < 0 Then m_Frequency = 0
                If m_Frequency > m_SamplesPerSec / 2.0F Then m_Frequency = m_SamplesPerSec / 2.0F
                Dim f As Int32 = CInt(m_oldfreq * (1 << 14))
                Dim df As Int32 = CInt(m_Frequency - m_oldfreq) * (1 << 14) \ data.Length
                m_oldfreq = m_Frequency
                Dim pmax As Int32 = m_SamplesPerSec << 14
                ' --------------------------------------------------------- fill buffer
                For i As Int32 = 0 To data.Length - 1
                    data(i) = m_WaveTable(m_phase >> 14)
                    m_phase += f
                    f += df
                    If m_phase >= pmax Then
                        m_phase -= pmax
                        ' Here starts a new cycle and it is possible to do updates 
                    End If
                Next
        End Select
    End Sub


    ' =====================================================================================
    '  PUBLIC FUNCTIONS
    ' =====================================================================================
    Friend Enum WaveTypes As Int32
        Sine = 0
        Triangle = 1
        RampPos = 2
        RampNeg = 3
        Square = 4
        Noise = 5
    End Enum

    Friend Sub PlayStart(Optional ByVal SamplesPerSec As Int32 = 44100)
        m_SamplesPerSec = SamplesPerSec
        InitAll()
        ao.OutputOn(0, m_SamplesPerSec, 16, 1, 1024, 16, AddressOf FillBuffer)
    End Sub

    Friend Sub PlayStart(ByVal freq As Single, _
                         ByVal db As Single, _
                         ByVal WaveType As WaveTypes, _
                         Optional ByVal SamplesPerSec As Int32 = 44100)

        m_SamplesPerSec = SamplesPerSec
        m_Frequency = freq
        m_OutLevel = db
        m_WaveType = WaveType
        InitAll()
        ao.OutputOn(0, m_SamplesPerSec, 16, 1, 1024, 16, AddressOf FillBuffer)
    End Sub

    Friend Sub SetFrequency(ByVal freq As Single)
        m_Frequency = freq
    End Sub

    Friend Sub SetOutLevel(ByVal dB As Single)
        If dB <> m_OutLevel Then
            m_OutLevel = dB
            InitAll()
        End If
    End Sub

    Friend Sub SetWaveType(ByVal wavetype As WaveTypes)
        If wavetype <> m_WaveType Then
            m_WaveType = wavetype
            InitAll()
        End If
    End Sub

    Friend Sub PlayStop()
        'ao.OutputPause()
        If ao IsNot Nothing Then ao.OutputOff()
    End Sub

End Class
