﻿
Class WavePlayer

    ' =====================================================================================
    '  PRIVATE VARS AND FUNCTIONS
    ' =====================================================================================
    Private m_Playing As Boolean = False
    Private m_Frequency As Single = 0
    Private m_oldfreq As Single
    Private m_OutLevel As Double = -120
    Private m_oldlevel As Double
    Private m_SamplesPerSec As Int32
    Private m_kdb As Double = 8.685889638
    Private m_phase As Int32
    Private m_WaveTable() As Single
    Private m_WaveType As WaveTypes
    Private m_randomInt As Int32
    Private m_randomFloat As Single
    Private ao As AudioOut = New AudioOut

    Private Sub InitAll()
        If m_WaveTable Is Nothing OrElse m_SamplesPerSec <> m_WaveTable.Length Then
            ReDim m_WaveTable(m_SamplesPerSec - 1)
            m_oldfreq = m_Frequency
            m_oldlevel = m_OutLevel
            m_phase = 0
        End If
        Select Case m_WaveType
            Case WaveTypes.Sine
                Dim k_pi As Double = (Math.PI * 2) / m_WaveTable.Length
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(Math.Sin(i * k_pi))
                Next
            Case WaveTypes.Triangle
                For i As Int32 = 0 To m_WaveTable.Length \ 4 - 1
                    m_WaveTable(i) = CSng(4 * i / m_WaveTable.Length)
                Next
                For i As Int32 = m_WaveTable.Length \ 4 To m_WaveTable.Length * 3 \ 4 - 1
                    m_WaveTable(i) = CSng(2 - (4 * i / m_WaveTable.Length))
                Next
                For i As Int32 = m_WaveTable.Length * 3 \ 4 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(4 * i / m_WaveTable.Length - 4)
                Next
            Case WaveTypes.RampPos
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(2 * (1 - i / m_WaveTable.Length) - 1)
                Next
            Case WaveTypes.RampNeg
                For i As Int32 = 0 To m_WaveTable.Length - 1
                    m_WaveTable(i) = CSng(2 * i / m_WaveTable.Length - 1)
                Next
            Case WaveTypes.Square
                For i As Int32 = 0 To m_WaveTable.Length \ 2 - 1
                    m_WaveTable(i) = 1
                Next
                For i As Int32 = m_WaveTable.Length \ 2 To m_WaveTable.Length - 1
                    m_WaveTable(i) = -1
                Next
        End Select
    End Sub

    Private Sub FillBuffer(ByRef data() As Int16)
        Select Case m_WaveType
            Case WaveTypes.Noise
                Dim kvol As Single = CSng(Math.Exp(m_OutLevel / m_kdb))
                If m_randomInt = 0 Then m_randomInt = 1
                For i As Int32 = 0 To data.Length - 1
                    ' --------------------------------------------------------- DAA random method
                    If (m_randomInt And &H800000) <> 0 Then
                        m_randomInt = (m_randomInt << 1) Xor &H1D872B41
                    Else
                        m_randomInt = m_randomInt << 1
                    End If
                    ' --------------------------------------------------------- limit randomFloat value
                    m_randomFloat += m_randomInt * 0.0002F
                    If Math.Abs(m_randomFloat) > 1000000000.0 Then m_randomFloat = 0
                    m_randomFloat = CInt(m_randomFloat) And 32767
                    ' --------------------------------------------------------- set the value
                    data(i) = CShort(kvol * m_randomFloat)
                Next
            Case Else
                ' ------------------------------------------------------------- limit frequency
                If m_Frequency < 0 Then m_Frequency = 0
                If m_Frequency > m_SamplesPerSec / 2.0F Then m_Frequency = m_SamplesPerSec / 2.0F
                ' ------------------------------------------------------------- prepare for smoot frequency change
                Dim f As Int32 = CInt(m_oldfreq * (1 << 14))
                Dim df As Int32 = CInt(m_Frequency - m_oldfreq) * (1 << 14) \ data.Length
                m_oldfreq = m_Frequency
                Dim pmax As Int32 = m_SamplesPerSec << 14
                ' ------------------------------------------------------------- prepare for smoot volume change
                Dim kvol As Single = CSng(32767 * Math.Exp(m_OutLevel / m_kdb))
                Dim oldkvol As Single = CSng(32767 * Math.Exp(m_oldlevel / m_kdb))
                Dim dv As Single = (kvol - oldkvol) / data.Length * 0.2F
                m_oldlevel += (m_OutLevel - m_oldlevel) * 0.2
                ' ------------------------------------------------------------- fill buffer
                For i As Int32 = 0 To data.Length - 1
                    data(i) = CShort(oldkvol * m_WaveTable(m_phase >> 14))
                    m_phase += f
                    f += df
                    oldkvol += dv
                    If m_phase >= pmax Then
                        m_phase -= pmax
                        ' Here starts a new cycle and it is possible to do updates 
                    End If
                Next
        End Select
    End Sub


    ' =====================================================================================
    '  PUBLIC FUNCTIONS
    ' =====================================================================================
    Friend Enum WaveTypes As Int32
        Sine = 0
        Triangle = 1
        RampPos = 2
        RampNeg = 3
        Square = 4
        Noise = 5
    End Enum

    Friend Sub SetWaveType(ByVal wavetype As WaveTypes)
        If wavetype <> m_WaveType Then
            m_WaveType = wavetype
            InitAll()
        End If
    End Sub

    Friend Sub SetFrequency(ByVal freq As Single)
        m_Frequency = freq
    End Sub

    Friend Sub SetOutLevel(ByVal dB As Single)
        If dB <> m_OutLevel Then
            m_OutLevel = dB
            InitAll()
        End If
    End Sub

    Friend Sub PlayStart(Optional ByVal SamplesPerSec As Int32 = 44100)
        m_SamplesPerSec = SamplesPerSec
        InitAll()
        ao.OutputOn(0, m_SamplesPerSec, 16, 1, 1024, 16, AddressOf FillBuffer)
        FadeIn()
        m_Playing = True
    End Sub

    Friend Sub PlayStart(ByVal freq As Single, _
                         ByVal db As Single, _
                         ByVal WaveType As WaveTypes, _
                         Optional ByVal SamplesPerSec As Int32 = 44100)
        m_Frequency = freq
        m_OutLevel = db
        m_WaveType = WaveType
        m_SamplesPerSec = SamplesPerSec
        InitAll()
        ao.OutputOn(0, m_SamplesPerSec, 16, 1, 1024, 16, AddressOf FillBuffer)

        FadeIn()
        m_Playing = True
    End Sub

    Friend Sub PlayStop()
        FadeOut()
        If ao IsNot Nothing Then ao.OutputOff()
        m_Playing = False
    End Sub

    Friend Sub FadeOut()
        If Not m_Playing Then Return
        Dim old As Double = m_OutLevel
        m_OutLevel = -120
        While Math.Abs(m_OutLevel - m_oldlevel) > 0.1
            Threading.Thread.Sleep(10)
            Application.DoEvents()
        End While
        m_OutLevel = old
    End Sub

    Friend Sub FadeIn()
        If m_Playing Then Return
        m_oldlevel = -120
        While Math.Abs(m_OutLevel - m_oldlevel) > 0.1
            Threading.Thread.Sleep(10)
            Application.DoEvents()
        End While
    End Sub

End Class

